import React, { useState, useRef, useEffect } from 'react';
import { CloseOutlined } from '@ant-design/icons';
import { Form, Input, Button, InputNumber, TreeSelect, Card, Space } from 'antd';
import { ProCard } from '@ant-design/pro-Card';

const GoodsAttribute = (props) => {
    const [form] = Form.useForm();
    const [expandedCardKeys, setExpandedCardKeys] = useState([]);
    const [inputValues, setInputValues] = useState([]);
    const [treeDatas, setTreeDatas] = useState([]);

    useEffect(() => {
        if (props.values != undefined) {
            if (Object.keys(props.values).length > 0) {
                var inputs = props.values.goodsAttributeViewList?.map((item, index) => {
                    return { index: index, value: item.attributeName }
                })
                setInputValues(inputs)
            }
        }
        else {
            setInputValues({ index: 0, value: '' })
        }
        const newData = props.treeDatas.map(item => ({ ...item, disabled: false }));
        var da = newData.find(t => t.key == props.values?.id);
        if (da) {
            da.disabled = true;
        }
        setTreeDatas(newData);
    }, []);
    const toggleContent = (key) => {
        if (expandedCardKeys.includes(key)) {
            setExpandedCardKeys(expandedCardKeys.filter(k => k !== key));
        } else {
            setExpandedCardKeys([...expandedCardKeys, key]);
        }
    };
    const handleInputChange = (e, field) => {
        // const updatedValues = [...inputValues];
        const updatedValues = [inputValues];
        updatedValues[field.key] = { ...updatedValues[field.key], value: e.target.value };
        setInputValues(updatedValues);
    };
    return (
        <Form
            labelCol={{
                span: 6,
            }}
            wrapperCol={{
                span: 18,
            }}
            form={form}
            name=""
            style={{
                maxWidth: 700,
            }}
            autoComplete="off"
            initialValues={{
                id: props.values?.id,
                pid: props.values?.pid,
                name: props.values?.name,
                remark: props.values?.remark,
                sort: props.values?.sort,
                attributeList: props.values?.goodsAttributeViewList
            }}
            onFinish={props.onFinish}
        >
            <Form.Item name="id" hidden={true}>
                <Input />
            </Form.Item>
            <Form.Item name="pid" label="上级名称" rules={[{ required: false }]} style={{ display: 'inline-block', width: '50%' }}>
                <TreeSelect
                    allowClear
                    showSearch
                    showCheckedStrategy='SHOW_PARENT'
                    style={{ width: '80%' }}
                    dropdownStyle={{ maxHeight: 400, overflow: 'auto' }}
                    treeData={treeDatas}
                    placeholder="请选择上级名称"
                    treeDefaultExpandAll
                />
            </Form.Item>
            <Form.Item name="name" label="类型名称" rules={[{ required: true, max: 50 }]} style={{ display: 'inline-block', width: '50%' }}>
                <Input style={{ width: '80%' }} />
            </Form.Item>
            <Form.Item name="remark" label="备注" style={{ display: 'inline-block', width: '50%' }}>
                <Input style={{ width: '80%' }} />
            </Form.Item>
            <Form.Item name="sort" label="排序" style={{ display: 'inline-block', width: '50%' }}>
                <InputNumber />
            </Form.Item>
            <Form.List name="attributeList">
                {(fields, { add, remove }) => (
                    <div
                        style={{
                            display: 'flex',
                            rowGap: 16,
                            flexDirection: 'column',
                        }}
                    >
                        {fields.map((field) => (
                            <ProCard
                                size="small"
                                title={((inputValues[field?.key]?.value == undefined ? '' : inputValues[field?.key]?.value)) + (!expandedCardKeys.includes(field.key) ? '(点击收缩)' : '(点击展开)')}
                                key={field.key}
                                type='inner'
                                headerBordered
                                bordered
                                collapsible
                                onClick={() => toggleContent(field.key)}
                                extra={
                                    <CloseOutlined
                                        onClick={() => {
                                            remove(field.name);
                                        }}
                                    />
                                }
                            >
                                {!expandedCardKeys.includes(field.key) && (
                                    <div onClick={(e) => e.stopPropagation()}>
                                        <Form.Item label="属性名称" name={[field.name, 'attributeName']} rules={[{ required: true, max: 50 }]} style={{ display: 'inline-block', width: '50%' }}>
                                            <Input placeholder="属性名称" onChange={(e) => handleInputChange(e, field)} />
                                        </Form.Item>
                                        <Form.Item label="序号" name={[field.name, 'sort']} rules={[{ required: true }]} style={{ display: 'inline-block', width: '50%' }}>
                                            <InputNumber placeholder="序号" />
                                        </Form.Item>
                                        <Form.Item label="属性值" rules={[{ required: true, max: 50 }]} style={{ display: 'inline-block', width: '100%', marginLeft: '-12.5%' }}>
                                            <Form.List name={[field.name, 'attributeValueList']}>
                                                {(subFields, subOpt) => (
                                                    <div
                                                        style={{
                                                            display: 'flex',
                                                            flexDirection: 'column',
                                                            rowGap: 16,
                                                        }}
                                                    >
                                                        {subFields.map((subField) => (
                                                            <Space key={subField.key}>
                                                                <Form.Item noStyle name={[subField.name, 'attributeValue']}>
                                                                    <Input placeholder="属性值" />
                                                                </Form.Item>
                                                                <Form.Item noStyle name={[subField.name, 'sort']}>
                                                                    <Input placeholder="序号" />
                                                                </Form.Item>
                                                                <CloseOutlined
                                                                    onClick={() => {
                                                                        subOpt.remove(subField.name);
                                                                    }}
                                                                />
                                                            </Space>
                                                        ))}
                                                        <Button type="dashed" onClick={() => subOpt.add()} block>
                                                            添加属性
                                                        </Button>
                                                    </div>
                                                )}
                                            </Form.List>
                                        </Form.Item>
                                    </div>
                                )}
                            </ProCard>
                        ))}
                        <Button type="dashed" onClick={() => add()} block>
                            添加属性值
                        </Button>
                    </div>
                )}
            </Form.List>
            <br />
            <Form.Item style={{ marginLeft: '70%' }}>
                <Button htmlType="button" style={{ float: 'right', left: 10 }} onClick={props.onCancel} >
                    取消
                </Button>
                <Button type="primary" htmlType="submit" style={{ float: 'right' }} >
                    保存
                </Button>
            </Form.Item>
        </Form>
    );
};

export default GoodsAttribute;
