import { PlusOutlined, UploadOutlined, QuestionCircleOutlined, DownloadOutlined } from '@ant-design/icons';
import { Button, message, Drawer, Popconfirm, Upload, Spin, Tooltip, Modal, Form } from 'antd';
import React, { useState, useRef, useEffect } from 'react';
import { PageContainer } from '@ant-design/pro-layout';
import ProTable from '@ant-design/pro-table';
import ProDescriptions from '@ant-design/pro-descriptions';
import { history } from 'umi';
import CreateForm from './components/CreateForm';
import ChooseForm from './components/ChooseForm';
import { getDataBaseUrl } from '@/global_data';
import { gettree, goods, removeGoods, addGoods, updateGoods, GetDeviceTechnology, GoodsExport, GetGoodsRecipeList } from './service';
import axios from 'axios';

const aliyunHostUrl = 'https://bpa.oss-cn-chengdu.aliyuncs.com/hkerp/test/goods/';

const GoodsManage = () => {
  const [goodsTypes, setGoodsTypes] = useState([]);
  /** 新建/更新窗口的弹窗 */
  const [createModalVisible, handleModalVisible] = useState(false);
  /** 分布更新窗口的弹窗 */
  const [form] = Form.useForm();
  const [showDetail, setShowDetail] = useState(false);
  //表单
  const myformRef = useRef();
  //列表
  const actionRef = useRef();
  const [currentRow, setCurrentRow] = useState();
  const [selectedRowsState, setSelectedRows] = useState([]);
  const [treeData, setTreeData] = useState([]);
  const [loading, setLoading] = useState(false);
  const [deviceData, setDeviceData] = useState([]);
  const [deviceId, setDeviceId] = useState('');
  const [deviceName, setDeviceName] = useState('');
  const [goodsId, setGoodsId] = useState([]);
  const [importFile, setImportFile] = useState();
  const [showChooseForm, setShowChooseForm] = useState(false);
  const [chooseFormData, setChooseFormData] = useState([]);
  const [showRecipeForm, setShowRecipeForm] = useState(false);
  const [recipeGoodsId, setRecipeGoodsId] = useState([]);
  /** 国际化配置 */
  useEffect(() => {
    gettree().then((r) => {
      setTreeData(r.data);
    });

    initGetDeviceTechnology();
  }, []);

  const initGetDeviceTechnology = () =>{
    GetDeviceTechnology().then((r) => {
      var arr = r.data;
      if (r.succeeded) {
        var list = [];
        arr.forEach((item) => {
          list.push({
            value: item.id, 
            label: item.deviceName
          });
        });
        setDeviceData(list);
      }
    });
  }

  /**
  * 添加节点
  *
  * @param fields
  */
  const handleAdd = async (fields) => {
    fields.price = fields.price ? fields.price : 0;
    fields.isAttrubute = true;
    fields.ImgUrl = fields.imgUrl1 ? aliyunHostUrl + fields.imgUrl1.file.name : null;
    await addGoods(JSON.stringify(fields)).then((r) => {
      if (r.succeeded) {
        message.success('添加成功');
        actionRef.current.reload();
      } else {
        message.error(r.errors);
      }
    });
    return true;
  };
/**
 * 更新节点去
 *
 * @param fields
 */
  const handleUpdate = async (fields) => {
    const hide = message.loading('正在修改');
    try {
      await updateGoods({
        id: fields.id,
        goodsTypeId: fields.goodsTypeId,
        name: fields.name,
        goodsUintId: fields.goodsUintId,
        price: fields.price ? fields.price : 0,
        isWeigh: fields.isWeigh,
        descritption: fields.descritption,
        imgUrl: fields.imgUrl1 ? aliyunHostUrl + fields.imgUrl1.file.name : fields.imgUrl
      }).then((r) => {
        if (r.succeeded) {
          message.success('修改成功');
          actionRef.current.reload();
        } else {
          message.error(r.errors);
        }
      });
      hide();
      return true;
    } 
    catch (error) {
      hide();
      message.error('修改失败请重试！');
      return false;
    }
  };
  const BatchDeletion = () => {
    return (
      selectedRowsState?.length > 0 &&
      (
        <Popconfirm
          title="确认删除吗?"
          onConfirm={onClickdele}
          okText="确定"
          cancelText="取消"
        >
          <Button type="primary">批量删除</Button>
        </Popconfirm>
      ))
  }
  const onClickdele = () => {
    handleRemove(selectedRowsState);
    setSelectedRows([]);
    actionRef.current?.reloadAndRest?.();
  }
  const columns = [
    {
      title: '主键',
      dataIndex: 'id',
      hideInTable: true,
      hideInSearch: true,
      tip: '规则名称是唯一的 key',
      render: (dom, entity) => {
        return (
          <a
            onClick={() => {
              setCurrentRow(entity);
              setShowDetail(true);
            }}
          >
            {dom}
          </a>
        );
      },
    },
    {
      title: '图片地址',
      dataIndex: 'imgUrl',
      hideInTable: true,
      hideInSearch: true,
    },
    {
      title: '名称',
      dataIndex: 'name',
      valueType: 'textarea',
    },
    {
      title: '商品价格',
      dataIndex: 'price',
      hideInForm: true,
      renderText: (val) => `${val}¥`,
      hideInSearch: true,
    },
    {
      title: '设备',
      dataIndex: 'deviceId',
      valueType: 'treeSelect',
      hideInTable: true,
      fieldProps: {
        showSearch:true,
        allowClear:true,
        options: deviceData,
        treeNodeFilterProp: 'label',
        onChange: (val, row) => {
          setDeviceId(val)
          setDeviceName(row[0])
          myformRef.current.submit()
        }
      },
    },
    {
      title: '工艺分类',
      dataIndex: 'goodsTypeName',
      valueType: 'treeSelect',
      fieldProps: {
        showSearch:true,
        allowClear:true,
        fieldNames: {
          children: 'children',
          label: 'title',
        },
        options: treeData,
        treeNodeFilterProp: 'title',
      },
    },
    {
      title: '备注',
      dataIndex: 'descritption',
      ellipsis: true,
      tip: '标题过长会自动收缩',
    },
    {
      title: '操作',
      dataIndex: 'option',
      valueType: 'option',
      fixed: 'right',
      width: 400,
      render: (_, record) => [
        <a
          key="primary"
          type="primary"
          onClick={() => {
            // history.push({
            //   pathname: '/database/goods/goodsInfo',
            //   query: {
            //     isAdd: false,
            //     values: record,
            //     tabStatus: 'basis'
            //   },
            // });
            handleModalVisible(true);
            setCurrentRow(record);
          }}
        >
          更新
        </a>,
        <a
          key="primary"
          type="primary"
          onClick={() => {
            history.push({
              pathname: '/database/goods/goodsbom',
              query: {
                isAdd: false,
                values: record,
                tabStatus: 'goodsbom'
              },
            });
            // handleModalVisible(true);
            // setCurrentRow(record);
          }}
        >
          配方配置
        </a>,
            <a
            key="primary"
            type="primary"
            onClick={() => {
              history.push({
                pathname: '/database/goods/goodstechnology',
                query: {
                  isAdd: false,
                  values: record,
                  tabStatus: 'goodstechnologymaken'
                },
              });
              // handleModalVisible(true);
              // setCurrentRow(record);
            }}
          >
            工艺配置
          </a>,
  //    <a
  //    key="primary"
  //    type="primary"
  //    onClick={() => {
  //      history.push({
  //        pathname: '/database/goods/goodsattributeprice',
  //        query: {
  //          isAdd: false,
  //          values: record,
  //        },
  //      });
  //      // handleModalVisible(true);
  //      // setCurrentRow(record);
  //    }}
  //  >
  //    价格配置
  //  </a>
        <a
          key="primary"
          type="primary"
          onClick={()=>{
            handleDownload(record.id)
          }}
        >
          下载商品
        </a>,
        <a
          key="primary"
          type="primary"
          onClick={() => {
            setRecipeGoodsId(record.id);
            setShowRecipeForm(true);
          }}
        >
          查看配方
        </a>,
        <Popconfirm
          type="primary"
          key="primary"
          title="确认删除吗？"
          okText="是"
          cancelText="否"
          onConfirm={() => {
            removeGoods(record.id).then((r) => {
              if (r.succeeded) {
                message.success('删除成功');
                actionRef.current.reload();
              } else {
                message.error(r.errors);
              }
            });
          }}
          onCancel={() => {}}
        >
          <a href="#">删除</a>
        </Popconfirm>,
      ],
    },
  ];
  const recipeColumns = [
    {
      title: '序号',
      dataIndex: 'sort',
      hideInSearch: true
    },
    {
      title: '商品名称',
      dataIndex: 'name',
      hideInSearch: true,
      render: (text) => text === "-" ? '' : text
    },
    {
      title: '工序名称',
      dataIndex: 'technologyName',
      hideInSearch: true,
      render: (text) => text === "-" ? '' : text
    },
    {
      title: '主/物料名称',
      dataIndex: 'batchingName',
      hideInSearch: true,
      render: (text) => text === "-" ? '' : text
    },
    {
      title: '重量',
      dataIndex: 'weight',
      hideInSearch: true,
      render: (text) => text === "-" ? '' : text
    },
  ];
  const importMessage = (json) =>{
    const hide = message.loading('正在导入');
    const formDataAll = new FormData();
    formDataAll.append('file', importFile);
    formDataAll.append('json', json);
    formDataAll.append('deviceId', deviceId);
    const headers = {
      'Content-Type': 'application/json',
      'Authorization': 'Bearer  ' + localStorage.getItem('token')
    };
    axios.post(getDataBaseUrl() + '/api/goods/goodsImport', formDataAll, { headers }).then(r => {
      if(r.data.succeeded == true){
        message.success("导入成功");
        actionRef.current.reload();
      }else{
        message.error(r.data.errors);
      }
    }).catch(error => {
      message.error(error);
    });
    hide();
  }

  /**
 * 删除节点
 *
 * @param selectedRows
 */
  const handleRemove = async (selectedRows) => {
    if (!selectedRows) return true;
    await removeGoods(selectedRows.map((row) => row.id)).then((r) => {
      if (r.data) {
        message.success('删除成功');
        actionRef.current.reload();
      } else {
        message.error('删除失败，请重试');
      }
    });
    return true;
  };
  const importProps = {
    beforeUpload: (file) => {
      setImportFile(file);
      if (deviceName == '' || deviceId == undefined) {
        message.error('请选择一个设备！');
        return  Upload.LIST_IGNORE;
      }
      if (
        file.type !== 'application/vnd.ms-excel' &&
        file.type !== 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' &&
        file.type !== 'application/x-zip-compressed'
      ) {
        message.error(`${file.name} 不是excel或zip文件`);
      }
      return file.type == 'application/vnd.ms-excel' ||
        file.type == 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' ||
        file.type == 'application/x-zip-compressed'
        ? true
        : Upload.LIST_IGNORE;
    },
    name: 'file',
    action: getDataBaseUrl() + '/api/goods/getFileToChoose',
    headers: {
      Authorization: 'Bearer  ' + localStorage.getItem('token')
    },
    onChange(info) {
      setLoading(true);
      if (info.file.status === 'done') {
        setLoading(false);
        if(info.file.response.data.length > 0){
          setChooseFormData(info.file.response.data);
          setShowChooseForm(true);
        }
        else{
          importMessage([]);
          myformRef.current.submit()
        }
      }
    //   setLoading(true);
    //   if (info.file.status !== 'uploading') {
    //     console.log(info.file, info.fileList);
    //   }
    //   if (info.file.status === 'done') {
    //     setLoading(false);
    //     message.success(`${info.file.name}导入成功`);
    //     actionRef.current.reload();
    //   } else if (info.file.status === 'error') {
    //     setLoading(false);
    //     message.error(`${info.file.name}:${info.file.response} `);
    //   }
    }
  };
  const handleDownload = async (goodId) => {
    if (deviceName == '' || deviceId == undefined) {
      message.error('请选择一个设备！');
      return;
    }
    const hide = message.loading('正在下载');
    await GoodsExport({ goodId: goodId, deviceId: deviceId, goodsId: goodsId }).then((r) => {
      if (r.succeeded) {
        const base64String = r.data;
        // 解码 Base64 数据
        const binaryString = atob(base64String);
        // 转换为 Uint8Array
        const bytes = new Uint8Array(binaryString.length);
        for (let i = 0; i < binaryString.length; i++) {
          bytes[i] = binaryString.charCodeAt(i);
        }
        // 创建 Blob 对象
        const blob = new Blob([bytes], { type: 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet' });
        // 创建一个临时的 URL，指向 Blob 对象
        const url = window.URL.createObjectURL(blob);
        // 创建 <a> 元素，用于下载文件
        const link = document.createElement('a');
        const fileName = "商品.zip";
        link.href = url;
        link.setAttribute('download', fileName);
        // 将 <a> 元素添加到页面中
        document.body.appendChild(link);
        // 模拟用户点击下载链接，触发文件下载
        link.click();
        // 下载完成后，移除 <a> 元素
        document.body.removeChild(link);
        message.success("下载成功");
      } else {
        message.error(r.errors);
      }
    })
    hide();
  }

  return (
    <PageContainer host header={{
      title: '',
      breadcrumb: {},
    }}>
      <Spin spinning={loading} tip="上传中...">
        <ProTable
          headerTitle=""
          formRef={myformRef}
          actionRef={actionRef}
          rowKey="id"
          pagination={{ defaultPageSize: 10 }}
          search={{
            labelWidth: 120,
          }}
          toolBarRender={() => [
            <Button
              type="primary"
              key="primary"
              onClick={() => {
                // history.push({
                //   pathname: '/database/goods/goodsInfo',
                //   query: {
                //     isAdd: true,
                //     values: null,
                //     tabStatus: 'basis'
                //   },
                // });
                handleModalVisible(true);
              }}
            >
              <PlusOutlined /> 新建
            </Button>,
            <Button
              key="primary"
              onClick={() => handleDownload(null)}
            >
              <DownloadOutlined />下载商品
              <Tooltip placement="top" title="下载后请使用wps进行编辑"><QuestionCircleOutlined /></Tooltip>
            </Button>,
            <Upload {...importProps} showUploadList={false}>
              <Button icon={<UploadOutlined />}>导入商品</Button>
            </Upload>,
            <BatchDeletion />
          ]}
          request={async (params) => {
            var goodsData = [];
            var goodsIds = [];
            if (params.goodsTypeId != undefined) {
              params.goodsTypeId = goodsTypes[params.goodsTypeId].goodsTypeId;
            }
            var total = 0;
            await goods(params).then((r) => {
              goodsData = r.data.data;
              goodsData.forEach((x) => {
                goodsIds.push(x.id);
                if (x.defaultMate != null) {
                  x.defaultMate = x.defaultMate.split(",");
                }
              });
              total = r.data.total;
            });
            setGoodsId(goodsIds);
            return {
              data: goodsData,
              success: true,
              total: total,
            };

          }}
          columns={columns}
          rowSelection={{
            onChange: (_, selectedRows) => {
              setSelectedRows(selectedRows);
            },
          }}
        />
        <CreateForm
          onFinish={async (value) => {
            var success = false;
            if (value.id) {
              success = await handleUpdate(value);
            } else {
              success = await handleAdd(value);
            }
            if (success) {
              handleModalVisible(false);
              setCurrentRow(undefined);
              if (actionRef.current) {
                actionRef.current.reload();
              }
            }
            actionRef.current.reload();
          }}
          onCancel={() => {
            handleModalVisible(false);
            setCurrentRow(undefined);
          }}
          createModalVisible={createModalVisible}
          values={currentRow || {}}
          form={form}
        />
        <Modal
          title="配方详情"
          open={showRecipeForm}
          onCancel={() => {
            setShowRecipeForm(false);
          }}
          width={1040}
          destroyOnClose={true}
          bodyStyle={{ padding: '32px 40px 1px 48px' }}
          footer={null}
          maskClosable={false}
        >
          <ProTable
            pagination={false}
            search={false}
            rowKey="sort"
            request={async () => {
              var recipeData = [];
              await GetGoodsRecipeList(recipeGoodsId).then((r) => {
                recipeData = r.data;
              });
              return {
                data: recipeData,
                success: true,
              };
            }}
            columns={recipeColumns}
          />
        </Modal>
        <ChooseForm 
          onFinish={async (data) => {
            const transformedData = Object.entries(data).map(([key, value]) => ({
              key,
              value
            }));
            var list = [];
            transformedData.forEach((item) => {
              list.push({
                key: item.key, 
                value: item.value == true ? true : false
              });
            });
            importMessage(JSON.stringify(list));
            setShowChooseForm(false);
            setChooseFormData(undefined);
            myformRef.current.submit()
          }}
          onCancel={() => {
            setShowChooseForm(false);
            setChooseFormData(undefined);
          }}
          showChooseForm={showChooseForm}
          values={chooseFormData || {}}
          form={form}
        />
        <Drawer
          width={600}
          visible={showDetail}
          onClose={() => {
            setCurrentRow(undefined);
            setShowDetail(false);
          }}
          closable={false}
        >
          {currentRow?.name && (
            <ProDescriptions
              column={2}
              title={currentRow?.name}
              request={async () => ({
                data: currentRow || {},
              })}
              params={{
                id: currentRow?.name,
              }}
              columns={columns}
            />
          )}
        </Drawer>
      </Spin>
    </PageContainer>
  );
};

export default GoodsManage;
